//=============================================================================
// JCScoreBoard
// A new coop scoreboard
//   will soon add monster scoring for the deathpawns
//=============================================================================
class JCScoreBoard extends UnrealScoreBoard;

struct SBEntry
{
	var		int		Ping;
	var		int		ID;
	var		bool	bSpectator;
	var		bool	bIsAdmin;
	var		string	PlayerName;
	var		int		EnemyKillCount;
	var		int		FriendKillCount;
	var		int		DeathCount;
	var		int		SuicideCount;
	var		int		Spree;
	var		float	Score;
};

var SBEntry SBTable[16];

//=============================================================================
function DrawHeader(canvas Canvas)
{
	local float XL, YL;
	local int step;
	local string ratio;
	local int i;

	if (Canvas.ClipX >= 640)
		step = 16;
	else
		step = 8;

	super.DrawHeader(Canvas);

	if(JCGRI(PlayerPawn(Owner).GameReplicationInfo).bCoopScore)
	{
		Canvas.SetPos(0, Canvas.ClipY/4);
		Canvas.bCenter= true;
		Canvas.Font = Canvas.LargeFont;
		Canvas.DrawText("PLAYERS:"@
			JCGRI(PlayerPawn(Owner).GameReplicationInfo).PlayerTotal@"    ENEMY:"@
			JCGRI(PlayerPawn(Owner).GameReplicationInfo).SkaarjTotal, false);
		Canvas.bCenter= false;
		Canvas.Font = RegFont;
	}
	// set drawing color to green
	Canvas.DrawColor.R = 0;
	Canvas.DrawColor.G = 255;
	Canvas.DrawColor.B = 0;

	Canvas.StrLen("9999", XL, YL);
	Canvas.SetPos(Canvas.ClipX/8 - XL - 8, Canvas.ClipY/8 * 3 - Step);
	Canvas.DrawText("Ping", false);

	if(JCPRI(PlayerPawn(Owner).PlayerReplicationInfo).bAdmin)
	{
		Canvas.SetPos(Canvas.ClipX/8, (Canvas.ClipY*3)/8 - Step);
		Canvas.DrawText("ID", false);
	}

	Canvas.SetPos(Canvas.ClipX/8 + 24, (Canvas.ClipY*3)/8 - Step);
	Canvas.DrawText("Name", false);

	Canvas.StrLen("999", XL, YL);
	Canvas.SetPos(Canvas.ClipX/2 - XL, (Canvas.ClipY*3)/8 - Step);
	Canvas.DrawText("Kills", false);

	Canvas.SetPos((Canvas.ClipX*19)/32 - XL, (Canvas.ClipY*3)/8 - Step);
	Canvas.DrawText("Deaths", false);

	Canvas.SetPos((Canvas.ClipX*11)/16 - XL, (Canvas.ClipY*3)/8 - Step);
	Canvas.DrawText("Spree", false);

	Canvas.SetPos((Canvas.ClipX*25)/32, (Canvas.ClipY*3)/8 - Step);
	Canvas.DrawText("Kill:Death");

	if( JCGRI(PlayerPawn(Owner).GameReplicationInfo).NumDownloaders>0 )
	{
		Canvas.DrawColor.R = 255;
		Canvas.SetPos(20, Canvas.ClipY-82);
		if( JCGRI(PlayerPawn(Owner).GameReplicationInfo).NumDownloaders>1 )
			Canvas.DrawText("There are currently"@
		JCGRI(PlayerPawn(Owner).GameReplicationInfo).NumDownloaders@"downloaders right now:");
		else Canvas.DrawText("There are currently"@
		JCGRI(PlayerPawn(Owner).GameReplicationInfo).NumDownloaders@"downloader right now:");
		for (i = 0; i< 4; i++)
		{
			if( JCGRI(PlayerPawn(Owner).GameReplicationInfo).Downloaders[i]!="" )
			{
				Canvas.SetPos(26, Canvas.ClipY-72+10*i);
				Canvas.DrawText("DL"@(i+1)@"-"@JCGRI(PlayerPawn(Owner).GameReplicationInfo).Downloaders[i]);
			}
		}
		Canvas.DrawColor.R = 0;
	}
} // DrawHeader()

function DrawTrailer(canvas Canvas)
{
	local int Hours, Minutes, Seconds;
	local string HourString, MinuteString, SecondString;
	local float XL, YL;

	if (Canvas.ClipX > 500)
	{
		Seconds = JCGRI(PlayerPawn(Owner).GameReplicationInfo).GameTime;
		Minutes = Seconds / 60;
		Hours   = Minutes / 60;
		Seconds = Seconds - (Minutes * 60);
		Minutes = Minutes - (Hours * 60);

		if (Seconds < 10)
			SecondString = "0"$Seconds;
		else
			SecondString = string(Seconds);

		if (Minutes < 10)
			MinuteString = "0"$Minutes;
		else
			MinuteString = string(Minutes);

		if (Hours < 10)
			HourString = "0"$Hours;
		else
			HourString = string(Hours);

		Canvas.bCenter = true;
		Canvas.StrLen("Test", XL, YL);
		Canvas.SetPos(0, Canvas.ClipY - 2*YL);
		Canvas.DrawText("Level Time: "$HourString$":"$MinuteString$":"$SecondString, true);
		Canvas.bCenter = false;
	}
	super.DrawTrailer(Canvas);
}

//=============================================================================
function DrawPing(canvas Canvas, int i, float XOffset, int LoopCount)
{
	local float XL, YL;
	local int step;

	if (Level.Netmode == NM_Standalone)
		return;

	if (Canvas.ClipX >= 640)
		step = 16;
	else
		step = 8;

	Canvas.StrLen(SBTable[I].Ping, XL, YL);
	Canvas.SetPos(Canvas.ClipX/8 - XL - 8, (Canvas.ClipY*3)/8 + (LoopCount * step));

	Canvas.Font = Font'TinyWhiteFont';
	Canvas.DrawText(SBTable[I].Ping, false);
	Canvas.Font = RegFont;
} // DrawPing()

//=============================================================================
function DrawID(canvas Canvas, int I, float XOffset, int LoopCount)
{
	local float XL, YL;
	local int step;

	if(!JCPRI(PlayerPawn(Owner).PlayerReplicationInfo).bAdmin) return;

	if (Canvas.ClipX >= 640)
		step = 16;
	else
		step = 8;

	Canvas.StrLen(SBTable[I].ID, XL, YL);

	Canvas.SetPos(Canvas.ClipX/8, (Canvas.ClipY*3)/8 + (LoopCount * Step));
	Canvas.DrawText(SBTable[I].ID, false);
} // DrawID()

//=============================================================================
function DrawName(canvas Canvas, int I, float XOffset, int LoopCount)
{
	local int step;

	if (Canvas.ClipX >= 640)
		step = 16;
	else
		step = 8;

	Canvas.SetPos(Canvas.ClipX/8 + 24, (Canvas.ClipY*3)/8 + (LoopCount * Step));
	Canvas.DrawText(SBTable[I].PlayerName, false);
} // DrawName()

//=============================================================================
function DrawKills(canvas Canvas, int I, float XOffset, int LoopCount)
{
	local int step;

	if (Canvas.ClipX >= 640)
		step = 16;
	else
		step = 8;

	Canvas.SetPos(Canvas.ClipX/2, (Canvas.ClipY*3)/8 + (LoopCount * step));

	if(SBTable[I].EnemyKillCount >= 100.0)
		Canvas.CurX -= 6.0;
	if(SBTable[I].EnemyKillCount >= 10.0)
		Canvas.CurX -= 6.0;

	Canvas.DrawText(SBTable[I].EnemyKillCount, false);
} // DrawKills()

//=============================================================================
function DrawDeaths(canvas Canvas, int I, float XOffset, int LoopCount)
{
	local int step;

	if (Canvas.ClipX >= 640)
		step = 16;
	else
		step = 8;

	// draw the death count
	Canvas.SetPos((Canvas.ClipX*19)/32, (Canvas.ClipY*3)/8 + (LoopCount * Step));
	if(SBTable[I].DeathCount >= 100.0)
		Canvas.CurX -= 6.0;
	if(SBTable[I].DeathCount >= 10.0)
		Canvas.CurX -= 6.0;

	Canvas.DrawText(SBTable[I].DeathCount, false);
} // DrawDeaths()

//=============================================================================
// Kill:Death ratio
function DrawSpree(canvas Canvas, int I, float XOffset, int LoopCount)
{
	local int step;

	if (Canvas.ClipX >= 640)
		step = 16;
	else
		step = 8;
	Canvas.SetPos((Canvas.ClipX*11)/16, (Canvas.ClipY*3)/8 + (LoopCount * Step));
	Canvas.DrawText(SBTable[I].Spree, false);
} // DrawSpree()

//=============================================================================
// Kill:Death ratio
function DrawScores(canvas Canvas, int I, float XOffset, int LoopCount)
{
	local int step;

	if (Canvas.ClipX >= 640)
		step = 16;
	else
		step = 8;

	Canvas.SetPos((Canvas.ClipX*25)/32, (Canvas.ClipY*3)/8 + (LoopCount * Step));
	if(SBTable[I].DeathCount == 0)
		Canvas.DrawText("N/A",false);
	else
		Canvas.DrawText(float(SBTable[I].EnemyKillCount) / float(SBTable[I].DeathCount), false);

} // DrawScores()

//=============================================================================
function SortScores(int n)
{
	local int i, j, max;

	for(i=0; i<n-1; i++ )
	{
		max = i;
		for ( j=i+1; j<N; j++ )
			if(SBTable[j].EnemyKillCount > SBTable[max].EnemyKillCount)
				max = j;
		Swap(max, i);
	}
} // SortScores

//=============================================================================
function Swap(int L, int R)
{
	local SBEntry tmpEntry;

	tmpEntry = SBTable[L];
	SBTable[L] = SBTable[R];
	SBTable[R] = tmpEntry;
	// swap two elements of these logical structures, for sorting
} // Swap()

//=============================================================================
function ShowScores(canvas Canvas)
{
	local PlayerReplicationInfo PRI;
	local PlayerPawn P;
	local int PlayerCount, LoopCount, i;
	local float XL, YL;

	Canvas.Font = RegFont;

	if(PlayerPawn(Owner).myHud != none && UnrealHUD(PlayerPawn(Owner).myHUD) != none)
	{
			UnrealHUD(PlayerPawn(Owner).myHUD).MOTDFadeOutTime = 128;
			UnrealHUD(PlayerPawn(Owner).myHUD).DrawMOTD(Canvas);
	}

	DrawHeader(Canvas);
	DrawTrailer(Canvas);

	// Wipe Everything
	for(i=0; i<ArrayCount(SBTable); i++)
	{
		SBTable[I].ID = 0;
		SBTable[I].bSpectator = false;
		SBTable[I].bIsAdmin = false;
		SBTable[I].EnemyKillCount = 0;
		SBTable[I].FriendKillCount = 0;
		SBTable[I].DeathCount = 0;
		SBTable[I].SuicideCount = 0;
		SBTable[I].Spree = 0;
		SBTable[I].Score = -500;
	}

	foreach AllActors (class'PlayerReplicationInfo', PRI)
	{
		SBTable[PlayerCount].Ping = PRI.Ping;
		SBTable[PlayerCount].PlayerName = PRI.PlayerName;
		SBTable[PlayerCount].Score = PRI.Score;
		if(JCPRI(PRI) != none) // in case of bots w/o JCPRI
		{
			SBTable[PlayerCount].ID = JCPRI(PRI).PlayerID;
			SBTable[PlayerCount].bSpectator = PRI.bIsSpectator;
//			if(JCPRI(PRI).bAdmin && JCGRI(PlayerPawn(Owner).GameReplicationInfo).bShowAdmins)
//				SBTable[PlayerCount].PlayerName = SBTable[PlayerCount].PlayerName @ "(admin)";
			if(JCGRI(PlayerPawn(Owner).GameReplicationInfo).bShowAdmins)
				SBTable[PlayerCount].bIsAdmin = JCPRI(PRI).bAdmin;
			SBTable[PlayerCount].EnemyKillCount = JCPRI(PRI).EnemyKillCount;
			SBTable[PlayerCount].FriendKillCount = JCPRI(PRI).FriendKillCount;
			SBTable[PlayerCount].DeathCount = JCPRI(PRI).DeathCount;
			SBTable[PlayerCount].SuicideCount = JCPRI(PRI).SuicideCount;
			SBTable[PlayerCount].Spree = JCPRI(PRI).jSpree;
		}
		PlayerCount++;
		if(PlayerCount >= ArrayCount(SBTable)) break;
	}

	SortScores(PlayerCount);
	LoopCount = 0;
	for(i=0; i<PlayerCount; i++)
	{
		Canvas.DrawColor.R = 255;
		Canvas.DrawColor.G = 255;
		Canvas.DrawColor.B = 255;
		DrawPing(Canvas, i, 0, LoopCount);

/*		set colors for display, this is priority order (low->high):
		default: green
		spectator: white
		admin: yellow
*/
		// default:green
		Canvas.DrawColor.R = 0;
		Canvas.DrawColor.G = 255;
		Canvas.DrawColor.B = 0;

		if(SBTable[I].bSpectator) // white
		{
			Canvas.DrawColor.R = 255;
			Canvas.DrawColor.G = 255;
			Canvas.DrawColor.B = 255;
		}
		if(SBTable[I].bIsAdmin) // yellow
		{
			Canvas.DrawColor.R = 255;
			Canvas.DrawColor.G = 255;
			Canvas.DrawColor.B = 0;
		}

		DrawID(Canvas, i, 0, LoopCount);
		DrawName(Canvas, i, 0, LoopCount);
		if(!SBTable[I].bSpectator)
		{
			DrawKills(Canvas, i, 0, LoopCount);
			DrawDeaths(Canvas, i, 0, LoopCount);
			DrawSpree(Canvas, i, 0, LoopCount);
			DrawScores(Canvas, i, 0, LoopCount);
		}
		LoopCount++;
	}

	Canvas.DrawColor.R = 255;
	Canvas.DrawColor.G = 255;
	Canvas.DrawColor.B = 255;

	// makes the MOTD show with the scoreboard
	if(PlayerPawn(Owner).myHud != none && UnrealHUD(PlayerPawn(Owner).myHUD) != none)
		UnrealHUD(PlayerPawn(Owner).myHUD).MOTDFadeOutTime = 0;
} // ShowScore()

//=============================================================================
// endit

defaultproperties
{
}
